import { isValidElement } from "react";
export class ExtendedSpeechSynthesis {
    static addToQueue(utterance) {
        this.queue.push(utterance);
    }
    static removeFromQueue(utterance) {
        if (!utterance)
            return this.queue.shift();
        this.queue = this.queue.filter((queuedUtterance) => queuedUtterance !== utterance);
    }
    static clearQueue() {
        this.queue = [];
    }
    static speakFromQueue() {
        const utterance = this.queue[0];
        if (utterance)
            speechSynthesis.speak(utterance);
    }
}
ExtendedSpeechSynthesis.queue = [];
export const getIndex = (parentIndex, index) => `${parentIndex === "" ? "" : parentIndex + "-"}${index}`;
export const sanitize = (text) => text.replace(/[;<>]/g, (match) => (match === ">" ? ")" : "("));
export function JSXToArray(element) {
    if (isValidElement(element)) {
        const { children } = element.props;
        if (Array.isArray(children))
            return children.map((child) => JSXToArray(child));
        return JSXToArray(children);
    }
    return typeof element === "string" ? element.split("") : typeof element === "number" ? [element.toString()] : [];
}
export function JSXToText(element) {
    if (isValidElement(element)) {
        const { children } = element.props;
        if (Array.isArray(children))
            return children.map((child) => JSXToText(child)).join(" ") + " ";
        return JSXToText(children);
    }
    return typeof element === "string" ? element : typeof element === "number" ? element.toString() : "";
}
export function findCharIndex(characters, index) {
    let currentIndex = 0;
    function recursiveSearch(array, parentIndex = "") {
        for (let i = 0; i < array.length; i++) {
            const element = array[i];
            if (Array.isArray(element)) {
                const result = recursiveSearch(element, i);
                if (result)
                    return getIndex(parentIndex, result);
            }
            else {
                currentIndex++;
                if (currentIndex > index)
                    return getIndex(parentIndex, i);
            }
        }
        currentIndex++;
        return "";
    }
    return recursiveSearch(characters);
}
export function isParent(index, parentIndex) {
    if (!(index === null || index === void 0 ? void 0 : index.startsWith(parentIndex)))
        return false;
    if (parentIndex) {
        const indexParts = index.split("-");
        const parentIndexParts = parentIndex.split("-");
        for (let i = 0; i < parentIndexParts.length; i++) {
            if (indexParts[i] !== parentIndexParts[i])
                return false;
        }
    }
    return true;
}
